@@ -32,17 +32,9 @@
 
 #include "viewer-render.h"
 
-#define DEFAULT_FONT_FAMILY "Sans"
-#define DEFAULT_FONT_SIZE 18
-
-#define _MAKE_FONT_NAME(family, size) family " " #size
-#define MAKE_FONT_NAME(family, size) _MAKE_FONT_NAME(family, size)
-
-const char *prog_name;
-
 gboolean opt_display = TRUE;
 int opt_dpi = 96;
-const char *opt_font = MAKE_FONT_NAME (DEFAULT_FONT_FAMILY, DEFAULT_FONT_SIZE);
+const char *opt_font = "";
 gboolean opt_header = FALSE;
 const char *opt_output = NULL;
 int opt_margin = 10;
@@ -68,6 +60,11 @@ const char *opt_pangorc = NULL;
 const PangoViewer *opt_viewer = NULL;
 const char *opt_language = NULL;
 gboolean opt_single_par = FALSE;
+PangoColor opt_fg_color = {0, 0, 0};
+guint16 opt_fg_alpha = 65535;
+gboolean opt_bg_set = FALSE;
+PangoColor opt_bg_color = {65535, 65535, 65535};
+guint16 opt_bg_alpha = 65535;
 
 /* Text (or markup) to render */
 static char *text;
@@ -80,25 +77,11 @@ fail (const char *format, ...)
   va_list vap;
   va_start (vap, format);
   msg = g_strdup_vprintf (format, vap);
-  g_printerr ("%s: %s\n", prog_name, msg);
+  g_printerr ("%s: %s\n", g_get_prgname (), msg);
 
   exit (1);
 }
 
-static PangoFontDescription *
-get_font_description (void)
-{
-  PangoFontDescription *font_description = pango_font_description_from_string (opt_font);
-
-  if ((pango_font_description_get_set_fields (font_description) & PANGO_FONT_MASK_FAMILY) == 0)
-    pango_font_description_set_family (font_description, DEFAULT_FONT_FAMILY);
-
-  if ((pango_font_description_get_set_fields (font_description) & PANGO_FONT_MASK_SIZE) == 0)
-    pango_font_description_set_size (font_description, DEFAULT_FONT_SIZE * PANGO_SCALE);
-
-  return font_description;
-}
-
 static PangoLayout *
 make_layout(PangoContext *context,
 	    const char   *text,
@@ -120,7 +103,7 @@ make_layout(PangoContext *context,
   pango_layout_set_single_paragraph_mode (layout, opt_single_par);
   pango_layout_set_wrap (layout, opt_wrap);
 
-  font_description = get_font_description ();
+  font_description = pango_font_description_from_string (opt_font);
   if (size > 0)
     pango_font_description_set_size (font_description, size * PANGO_SCALE);
 
@@ -156,7 +139,7 @@ make_layout(PangoContext *context,
 gchar *
 get_options_string (void)
 {
-  PangoFontDescription *font_description = get_font_description ();
+  PangoFontDescription *font_description = pango_font_description_from_string (opt_font);
   gchar *font_name;
   gchar *result;
 
@@ -365,7 +348,7 @@ parse_enum (GType       type,
 	    int        *value,
 	    const char *name,
 	    const char *arg,
-	    gpointer    data,
+	    gpointer    data G_GNUC_UNUSED,
 	    GError **error)
 {
   char *possible_values = NULL;
@@ -385,7 +368,6 @@ parse_enum (GType       type,
 		  "Argument for %s must be one of %s",
 		  name,
 		  possible_values);
-      ret = FALSE;
     }
 
   g_free (possible_values);
@@ -399,7 +381,7 @@ parse_align (const char *name,
 	     gpointer    data,
 	     GError **error)
 {
-  return parse_enum (PANGO_TYPE_ALIGNMENT, &opt_align,
+  return parse_enum (PANGO_TYPE_ALIGNMENT, (int*)(void*)&opt_align,
 		     name, arg, data, error);
 }
 
@@ -409,7 +391,7 @@ parse_ellipsis (const char *name,
 		gpointer    data,
 		GError **error)
 {
-  return parse_enum (PANGO_TYPE_ELLIPSIZE_MODE, &opt_ellipsize,
+  return parse_enum (PANGO_TYPE_ELLIPSIZE_MODE, (int*)(void*)&opt_ellipsize,
 		     name, arg, data, error);
 }
 
@@ -419,7 +401,7 @@ parse_gravity (const char *name,
 	       gpointer    data,
 	       GError **error)
 {
-  return parse_enum (PANGO_TYPE_GRAVITY, &opt_gravity,
+  return parse_enum (PANGO_TYPE_GRAVITY, (int*)(void*)&opt_gravity,
 		     name, arg, data, error);
 }
 
@@ -429,14 +411,14 @@ parse_gravity_hint (const char *name,
 		    gpointer    data,
 		    GError **error)
 {
-  return parse_enum (PANGO_TYPE_GRAVITY_HINT, &opt_gravity_hint,
+  return parse_enum (PANGO_TYPE_GRAVITY_HINT, (int*)(void*)&opt_gravity_hint,
 		     name, arg, data, error);
 }
 
 static gboolean
-parse_hinting (const char *name,
+parse_hinting (const char *name G_GNUC_UNUSED,
 	       const char *arg,
-	       gpointer    data,
+	       gpointer    data G_GNUC_UNUSED,
 	       GError    **error)
 {
   gboolean ret = TRUE;
@@ -466,7 +448,7 @@ parse_wrap (const char *name,
 	    GError    **error)
 {
   gboolean ret;
-  if ((ret = parse_enum (PANGO_TYPE_WRAP_MODE, &opt_wrap,
+  if ((ret = parse_enum (PANGO_TYPE_WRAP_MODE, (int*)(void*)&opt_wrap,
 			 name, arg, data, error)))
     {
       opt_wrap_set = TRUE;
@@ -474,6 +456,92 @@ parse_wrap (const char *name,
   return ret;
 }
 
+static gboolean
+parse_rgba_color (PangoColor *color,
+		  guint16    *alpha,
+		  const char *name,
+		  const char *arg,
+		  gpointer    data G_GNUC_UNUSED,
+		  GError    **error)
+{
+  gboolean ret;
+  char buf[32];
+  int len;
+
+  len = strlen (arg);
+  /* handle alpha */
+  if (*arg == '#' && (len == 5 || len == 9 || len == 17))
+    {
+      int width, bits;
+      unsigned int a;
+
+      bits = len - 1;
+      width = bits >> 2;
+
+      strcpy (buf, arg);
+      arg = buf;
+
+      if (!sscanf (buf + len - width, "%x", &a))
+        {
+	  ret = FALSE;
+	  goto err;
+	}
+      buf[len - width] = '\0';
+
+      a <<= (16 - bits);
+      while (bits < 16)
+        {
+	  a |= (a >> bits);
+	  bits *= 2;
+	}
+      *alpha = a;
+    }
+  else
+    *alpha = 65535;
+
+  ret = pango_color_parse (color, arg);
+
+err:
+  if (!ret && error)
+    {
+      g_set_error(error,
+		  G_OPTION_ERROR,
+		  G_OPTION_ERROR_BAD_VALUE,
+		  "Argument for %s must be a color name like red, or CSS-style #rrggbb / #rrggbbaa",
+		  name);
+    }
+
+  return ret;
+}
+
+static gboolean
+parse_foreground (const char *name,
+		  const char *arg,
+		  gpointer    data,
+		  GError **error)
+{
+  return parse_rgba_color (&opt_fg_color, &opt_fg_alpha,
+			   name, arg, data, error);
+}
+
+static gboolean
+parse_background (const char *name,
+		  const char *arg,
+		  gpointer    data,
+		  GError **error)
+{
+  opt_bg_set = TRUE;
+
+  if (0 == strcmp ("transparent", arg))
+    {
+      opt_bg_alpha = 0;
+      return TRUE;
+    }
+
+  return parse_rgba_color (&opt_bg_color, &opt_bg_alpha,
+			   name, arg, data, error);
+}
+
 static gchar *
 backends_to_string (void)
 {
@@ -523,9 +591,9 @@ backend_description (void)
 }
 
 static gboolean
-parse_backend (const char *name,
+parse_backend (const char *name G_GNUC_UNUSED,
 	       const char *arg,
-	       gpointer    data,
+	       gpointer    data G_GNUC_UNUSED,
 	       GError    **error)
 {
   gboolean ret = TRUE;
@@ -555,16 +623,16 @@ parse_backend (const char *name,
 
 
 static gboolean
-show_version(const char *name,
-	     const char *arg,
-	     gpointer    data,
-	     GError    **error)
+show_version(const char *name G_GNUC_UNUSED,
+	     const char *arg G_GNUC_UNUSED,
+	     gpointer    data G_GNUC_UNUSED,
+	     GError    **error G_GNUC_UNUSED)
 {
-  g_printf("%s (%s) %s\n", prog_name, PACKAGE_NAME, PACKAGE_VERSION);
-  g_printf("module interface version: %s\n", MODULE_VERSION);
+  g_printf("%s (%s) %s\n", g_get_prgname (), PACKAGE_NAME, PACKAGE_VERSION);
+  g_printf("\nPango module interface version: %s\n", MODULE_VERSION);
 
   if (PANGO_VERSION != pango_version())
-    g_printf("\nLinked Pango library has a different version: %s\n", pango_version_string ());
+    g_printf("Linked Pango library has a different version: %s\n", pango_version_string ());
 
   exit(0);
 }
@@ -581,6 +649,8 @@ parse_options (int argc, char *argv[])
      "No layout direction according to contents",			NULL},
     {"backend",		0, backend_flag, G_OPTION_ARG_CALLBACK,		&parse_backend,
      backend_desc,					     backend_options},
+    {"background",	0, 0, G_OPTION_ARG_CALLBACK,			&parse_background,
+     "Set the background color",     "red/#rrggbb/#rrggbbaa/transparent"},
     {"no-display",	'q', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE,	&opt_display,
      "Do not display (just write to file or whatever)",			NULL},
     {"dpi",		0, 0, G_OPTION_ARG_INT,				&opt_dpi,
@@ -591,6 +661,8 @@ parse_options (int argc, char *argv[])
      "Ellipsization mode",				  "start/middle/end"},
     {"font",		0, 0, G_OPTION_ARG_STRING,			&opt_font,
      "Set the font description",			       "description"},
+    {"foreground",	0, 0, G_OPTION_ARG_CALLBACK,			&parse_foreground,
+     "Set the text color",		         "red/#rrggbb/#rrggbbaa"},
     {"gravity",		0, 0, G_OPTION_ARG_CALLBACK,			&parse_gravity,
      "Base gravity: glyph rotation",		"south/east/north/west/auto"},
     {"gravity-hint",	0, 0, G_OPTION_ARG_CALLBACK,			&parse_gravity_hint,
@@ -639,10 +711,19 @@ parse_options (int argc, char *argv[])
   GError *parse_error = NULL;
   GOptionContext *context;
   size_t len;
+  const PangoViewer **viewer;
 
-  prog_name = g_path_get_basename (argv[0]);
   context = g_option_context_new ("- FILE");
   g_option_context_add_main_entries (context, entries, NULL);
+
+  for (viewer = viewers; *viewer; viewer++)
+    if ((*viewer)->get_option_group)
+      {
+        GOptionGroup *group = (*viewer)->get_option_group (*viewer);
+	if (group)
+	  g_option_context_add_group (context, group);
+      }
+
   if (!g_option_context_parse (context, &argc, &argv, &parse_error))
   {
     if (parse_error != NULL)
@@ -660,7 +741,7 @@ parse_options (int argc, char *argv[])
       if (opt_text && argc != 1)
 	fail ("When specifying --text, no file should be given");
 
-      g_printerr ("Usage: %s [OPTION...] FILE\n", prog_name);
+      g_printerr ("Usage: %s [OPTION...] FILE\n", g_get_prgname ());
       exit (1);
     }
 
